/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.storage.eclipse;

import java.util.Collection;

import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.transaction.RecordingCommand;
import org.eclipse.emf.transaction.RollbackException;
import org.eclipse.emf.transaction.Transaction;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.emf.transaction.impl.InternalTransactionalEditingDomain;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.utils.LoggingUtils;

/**
 * This class wraps an EMF command and makes sure that all execute methods ( {@link #execute()},
 * {@link #undo()}, {@link #redo()}) are executed inside of
 * a transaction. As a bonus the undo and redo methods are based on the
 * transaction of the execute call (i.e. are coming for free).
 * 
 * @author hummel
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 1071B3466B824522BF38DE9AC367D70F
 */
public class EMFTransactionalCommand implements Command {

	/** The wrapped command. */
	private final Command inner;

	/** The recording command used. */
	private final RecordingCommand recordingCommand;

	/** The editing domain. */
	private final InternalTransactionalEditingDomain editingDomain;

	/** Constructor. */
	public EMFTransactionalCommand(Command command, TransactionalEditingDomain editingDomain) {
		inner = command;
		recordingCommand = new RecordingCommand(editingDomain) {
			@Override
			protected void doExecute() {
				inner.execute();
			}
		};
		this.editingDomain = (InternalTransactionalEditingDomain)editingDomain;
	}

	/** {@inheritDoc} */
	@Override
	public void execute() {
		runInTransaction(new Runnable() {
			@Override
			public void run() {
				recordingCommand.execute();
			}
		});
	}

	/** {@inheritDoc} */
	@Override
	public void redo() {
		runInTransaction(new Runnable() {
			@Override
			public void run() {
				recordingCommand.redo();
			}
		});
	}

	/** {@inheritDoc} */
	@Override
	public void undo() {
		runInTransaction(new Runnable() {
			@Override
			public void run() {
				recordingCommand.undo();
			}
		});
	}

	/** Executes the given runnable in a transaction. */
	private void runInTransaction(Runnable runnable) {
		Transaction tx = null;
		try {
			tx = editingDomain.startTransaction(false, null);
			runnable.run();
		} catch(InterruptedException e) {
			LoggingUtils.error(ToolingKernelActivator.getDefault(),
					"Had unexpected interruption in command execution!", e);
		} finally {
			if(tx != null) {
				try {
					tx.commit();
				} catch(RollbackException e) {
					LoggingUtils.error(ToolingKernelActivator.getDefault(),
							"Transaction was rolled back!", e);
				}
			} else {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						"Was not able to start transaction!");
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public boolean canExecute() {
		return inner.canExecute();
	}

	/** {@inheritDoc} */
	@Override
	public boolean canUndo() {
		return recordingCommand.canUndo();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<?> getAffectedObjects() {
		return inner.getAffectedObjects();
	}

	/** {@inheritDoc} */
	@Override
	public String getDescription() {
		return inner.getDescription();
	}

	/** {@inheritDoc} */
	@Override
	public String getLabel() {
		return inner.getLabel();
	}

	/** {@inheritDoc} */
	@Override
	public Collection<?> getResult() {
		return inner.getResult();
	}

	/** {@inheritDoc} */
	@Override
	public Command chain(Command command) {
		if(command == null) {
			return this;
		}
		CompoundCommand result = new CompoundCommand();
		result.append(this);
		result.append(command);
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		inner.dispose();
		recordingCommand.dispose();
	}
}
