package org.fortiss.tooling.kernel.service;

import java.util.List;

import org.fortiss.tooling.kernel.extension.IConstraintVerifier;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.internal.ConstraintVerificationService;
import org.fortiss.tooling.kernel.model.constraints.ConstraintInstance;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.OutdatedVerificationStatus;

/**
 * The verification service allows to register "constraint verifiers", e.g.,
 * a test suite runner, a temporal logic verifier, a WCET checker, a requirement review system.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 88EE1A87BCC43DE314404EE52B3D98A7
 */
public interface IConstraintVerificationService {
	/** Returns the service instance. */
	public static IConstraintVerificationService getInstance() {
		return ConstraintVerificationService.getInstance();
	}

	/**
	 * @param ci
	 *            Verifies the constraint instance <code>ci</code>. The framework always runs the
	 *            verification in a command. Should not yield any "useless" change because
	 *            verification should always update the constraint instance status and the checksums
	 *            anyways.
	 */
	void verify(ConstraintInstance ci);

	/**
	 * @param ci
	 *            Cancels the last verification of the constraint instance <code>ci</code>.
	 */
	void cancel(ConstraintInstance ci);

	/**
	 * @param ci
	 * @return <code>true</code> if the constraint instance <code>ci</code> is up to date.
	 *         This does not just look at the status but computes a diff between the constrained
	 *         element and the one for which the constraint instance was last time checked. In the
	 *         standard case, it is enough to call IConstraintVerificationUIService.getStatus and
	 *         see if the result is an instance of {@link OutdatedVerificationStatus}. You should
	 *         use {isUpToDate} *only if IConstraintVerificationUIService.getStatus does not provide
	 *         the information you want*.
	 */
	boolean isUpToDate(ConstraintInstance ci);

	/**
	 * @param ci
	 * @return A list of possible automatic fixes for <code>ci</code>. A constraint instance should
	 *         be candidate to fixing if it is anything but successful or outdated.
	 */
	List<IFix> fixes(ConstraintInstance ci);

	/** Interface for a "fix". */
	public interface IFix {
		/**
		 * Description of the fix. Useful to allow the user distinguish the fix in case there are
		 * several.
		 */
		String getDescription();

		/**
		 * @param status
		 *            Fixes (the constraint instance of) <code>status</code>.
		 */
		void runFix(IConstraintVerificationStatus status);
	}

	/**
	 * @param top
	 * @param id
	 *            Completes <code>top</code> with missing constraints instances of constraint
	 *            <code>id</code>, if applicable.
	 * @return List of added constraints instances.
	 * 
	 */
	List<ConstraintInstance> addMissingConstraintsInstances(ITopLevelElement top, String id);

	/**
	 * @param constraint
	 * @param constrained
	 * @param index
	 *            Defines <code>constrained</code> as the <code>index</code>nth constrained model
	 *            element of <code>constraint</code>.
	 *            Leaves the other elements as they are if they exist, fill them with
	 *            <code>null</code> till <code>index</code> otherwise.
	 */
	void setConstrainedElement(ConstraintInstance constraint, IConstrained constrained, int index);

	/** Registers the given verifier with the service. */
	// TODO: change interface (see tutorial service for implementation. */
	// void registerConstraintVerifier(Class<? extends IConstraintVerifier> verifier);
	void registerConstraintVerifier(IConstraintVerifier verifier);
}
