/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.widget;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.fortiss.tooling.base.ui.utils.GCUtils.GCStateManager;
import org.fortiss.tooling.spiderchart.model.DataSeries;
import org.fortiss.tooling.spiderchart.model.SpiderChart;
import org.fortiss.tooling.spiderchart.style.ChartStyle;
import org.fortiss.tooling.spiderchart.style.FontStyle;
import org.fortiss.tooling.spiderchart.style.LegendStyle;
import org.fortiss.tooling.spiderchart.style.LineStyle;

/**
 * Class used for drawing the the spider chart legend.
 *
 * @author mondal
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: B24692F23825CB27B6F4E552CF509981
 */
public final class SpiderChartLegendWidget extends SpiderChartWidgetBase {
	/** The length of the line for each data series. */
	private static final int LINE_SIZE = 20;
	/** The space between line and text. */
	private static final int LINE_SPACE = 3;

	/** Click areas of the legend entries. */
	private final Map<DataSeries, Rectangle> clickAreas = new HashMap<DataSeries, Rectangle>();

	/** Constructor. */
	public SpiderChartLegendWidget(SpiderChart chart, ChartStyle style) {
		super(chart, style);
	}

	/** Draws the provided graphics */
	public void draw(GCStateManager gcState) {
		if(!style.isShowLegend()) {
			return;
		}
		if(style.getLegendStyle().isVerticalLayout()) {
			drawVertical(gcState);
		} else {
			drawHorizontal(gcState);
		}
	}

	/** draws the graphics horizontally */
	public void drawHorizontal(GCStateManager gcState) {
		String labelText = chart.getLegendLabel();
		int margin = style.getLegendStyle().getMargin();
		GC gc = gcState.GC();

		int xDelta = 0;
		if(labelText != null && labelText.trim().length() > 0) {
			xDelta = drawHorizontalLabel(gcState);
		}
		int y = getY() + margin;
		for(DataSeries data : chart.getDataSeries()) {
			String text = data.getName();
			Point extent = gc.textExtent(text);
			LineStyle l = style.getDataSeriesStyle(data).getLineStyle();
			gcState.foregroundStoreAndCreate(l.getRGBColor());

			int xl = getX() + xDelta;
			l.draw(gcState, xl, y + extent.y / 2, xl + LINE_SIZE, y + extent.y / 2);
			xDelta = xDelta + LINE_SIZE + LINE_SPACE;

			int xs = getX() + xDelta;
			gc.drawString(text, xs, y, true);
			xDelta = xDelta + extent.x + margin;
			gcState.foregroundRestoreAndDispose();

			Rectangle clickArea = new Rectangle(xl, y, xDelta - xl, extent.y);
			clickAreas.put(data, clickArea);
		}
	}

	/** Draws the horizontal legend label and returns the offset. */
	private int drawHorizontalLabel(GCStateManager gcState) {
		String labelText = chart.getLegendLabel();
		LegendStyle legendStyle = style.getLegendStyle();
		FontStyle labelStyle = legendStyle.getLabelStyle();
		int margin = legendStyle.getMargin();
		GC gc = gcState.GC();

		gcState.fontStoreAndCreate(labelStyle.getFontData());
		gcState.foregroundStoreAndCreate(labelStyle.getRGBColor());

		gc.setClipping(getX(), getY(), getWidth(), getHeight());
		Point textExtent = gc.textExtent(labelText);
		gc.drawString(labelText, getX() + margin, getY() + margin, true);
		gc.setClipping((Rectangle)null);

		gcState.foregroundRestoreAndDispose();
		gcState.fontRestoreAndDispose();

		return textExtent.x + 2 * margin;
	}

	/** draws the graphics vertically */
	public void drawVertical(GCStateManager gcState) {
		String labelText = chart.getLegendLabel();
		LegendStyle legendStyle = style.getLegendStyle();
		int margin = legendStyle.getMargin();
		GC gc = gcState.GC();

		int yDelta = 0;
		if(labelText != null && labelText.trim().length() > 0) {
			yDelta = drawVerticalLabel(gcState);
		}

		int textWidth = 0;
		int textHeight = 0;

		for(DataSeries data : chart.getDataSeries()) {
			Point extent = gc.textExtent(data.getName());
			textWidth = Math.max(textWidth, extent.x);
			textHeight = Math.max(textHeight, extent.y);
		}

		int cnt = 0;
		for(DataSeries data : chart.getDataSeries()) {
			int dY = yDelta + cnt * (textHeight + margin);
			int dX = LINE_SIZE + 2 * margin;
			LineStyle l = style.getDataSeriesStyle(data).getLineStyle();
			gcState.foregroundStoreAndCreate(l.getRGBColor());

			int lY = dY + textHeight / 2;
			l.draw(gcState, getX() + margin, getY() + lY, getX() + margin + LINE_SIZE, getY() + lY);
			gc.drawText(data.getName(), getX() + dX, getY() + dY, true);

			gcState.foregroundRestoreAndDispose();
			cnt++;

			int totalWidth = margin + LINE_SIZE + textWidth;
			Rectangle clickArea = new Rectangle(getX(), getY() + dY, totalWidth, textHeight);
			clickAreas.put(data, clickArea);
		}
	}

	/** Draws the header label for the vertical legend layout and returns the offset. */
	private int drawVerticalLabel(GCStateManager gcState) {
		String labelText = chart.getLegendLabel();
		LegendStyle legendStyle = style.getLegendStyle();
		FontStyle labelStyle = legendStyle.getLabelStyle();
		int margin = legendStyle.getMargin();
		GC gc = gcState.GC();

		gcState.fontStoreAndCreate(labelStyle.getFontData());
		gcState.foregroundStoreAndCreate(labelStyle.getRGBColor());

		gc.setClipping(getX(), getY(), getWidth(), getHeight());
		Point textExtent = gc.textExtent(labelText);
		int x = getX() + margin + (getWidth() - margin - textExtent.x) / 2;
		gc.drawString(labelText, x, getY() + margin, true);
		gc.setClipping((Rectangle)null);

		gcState.foregroundRestoreAndDispose();
		gcState.fontRestoreAndDispose();

		return textExtent.y + margin;
	}

	/**
	 * Scans the legend entries for a hit at the given point and returns the corresponding
	 * {@link DataSeries}.
	 */
	public DataSeries legendEntryHitScan(int x, int y) {
		for(DataSeries d : clickAreas.keySet()) {
			Rectangle a = clickAreas.get(d);
			if(a != null && a.contains(x, y)) {
				return d;
			}
		}
		return null;
	}
}
